import asyncio
from typing import Optional, Callable
from httpx import AsyncClient
from requests_oauthlib import OAuth2Session
from async_oauthlib import OAuth2Session as AsyncOAuth2Session
import requests
from .RequestException import EtsyRequestException
from etsy_apiv3.models import Me


class EtsySession:

    def __token_updater(self, token):
        return token

    async def __async_token_saver(self, token):
        return token
    
    def __init__(self,
                 client_key: str,
                 client_secret: str,
                 token: Optional[dict] = None,
                 token_updater: Optional[Callable] = None
                 ):

        self.CLIENT_KEY = client_key
        self.CLIENT_SECRET = client_secret
        self.__base_endpoint = "https://openapi.etsy.com/v3/application/"
        self.__oauth2_session = None
        
        if token is not None:
            if token_updater is None:
                self.token_updater = self.__token_updater
            else:
                self.token_updater = token_updater
            self.TOKEN = token
            
            self.__refresh_url = "https://api.etsy.com/v3/public/oauth/token"
            self.__oauth2_session = self.__create_oath2_session()
            
            self.__me = None
            
        else:
            self.__oauth2_session = requests.Session()

    def __create_oath2_session(self) -> OAuth2Session:
        refresh_kwargs = {
            'client_id': self.CLIENT_KEY,
            'client_secret': self.CLIENT_SECRET,
        }

        session = OAuth2Session(self.CLIENT_KEY, token=self.TOKEN, auto_refresh_kwargs=refresh_kwargs,
                                auto_refresh_url=self.__refresh_url, token_updater=self.token_updater
                                )
        return session
    
    def create_async_oauth2_session(self) -> AsyncOAuth2Session:
        refresh_kwargs = {
        'client_id': self.CLIENT_KEY,
        'client_secret': self.CLIENT_SECRET,
        }
        session = AsyncOAuth2Session(self.CLIENT_KEY, token=self.TOKEN, auto_refresh_kwargs=refresh_kwargs,
            auto_refresh_url=self.__refresh_url, token_updater=self.__async_token_saver
        )
        return session
    
    @property
    def me(self) -> Me:
        if self.__me is None:
            self.__me = Me(**self.request(f"users/me"))
        return self.__me
    
    @staticmethod
    def create_response(response: requests.Response):
        json: dict = response.json()

        if "error" in json.keys():
            raise EtsyRequestException(response.status_code, json["error"])
        
        return json

    def request(self, endpoint: str, method="GET", headers: Optional[dict] = None, *args, **kwargs) -> dict:
        """ 
        Send Request to target endpoint by method

        Args:
            endpoint (str): Api Endpoint Url
            method (str, optional): HTTP Methods [GET, POST, PUT, DELETE, UPDATE]. Defaults to "GET".

        Raises:
            EtsyRequestException: EtsyRequestException(status_code, message)

        Returns:
            json: Json From Request Response
        """
        if headers is None:
            headers = {"x-api-key": self.CLIENT_KEY}

        url = f"{self.__base_endpoint}{endpoint}"
        req = self.__oauth2_session.request(
            method, url, headers=headers, *args, **kwargs)

        return self.create_response(req)

    async def async_request(self, endpoint: str,  session: AsyncOAuth2Session = None, method="GET", headers: Optional[dict] = None, *args, **kwargs) -> dict:
        
        url = f"{self.__base_endpoint}{endpoint}"
        
        if headers is None:
            headers = {"x-api-key": self.CLIENT_KEY}
        
        res = await session.get(url=url, headers=headers, *args, **kwargs)
        json = await res.json()
        if "error" in json.keys():
            raise EtsyRequestException(res.status, json["error"])
        return json



from __future__ import annotations
from typing import List, Optional
from pydantic import BaseModel

class ListingProperty(BaseModel):
    property_id: int
    property_name: str
    scale_id: Optional[int]
    scale_name: Optional[str]
    value_ids: List[int]
    values: List[str]


from typing import Optional
from pydantic import BaseModel


class ReturnPolicy(BaseModel):
    return_policy_id: int
    shop_id: int
    accepts_returns: bool
    accepts_exchanges: bool
    return_deadline: Optional[int] = None
    


from dataclasses import dataclass
#from etsy_apiv3.utils.APIV3 import EtsyAuth
from async_oauthlib import OAuth2Session as AsyncOAuth2Session
from etsy_apiv3.utils import Response, EtsySession, EtsyOauth2Session
from ..models.ReviewModel import Review
from typing import Union

@dataclass
class ReviewResource:
    session: Union[EtsySession, EtsyOauth2Session]
    
    def get_reviews_by_listing_id(self, listing_id: int, limit: int = 25, offset: int = 0, min_created: int = 946684800, max_created: int = None):
        
        params = {"limit": limit, "offset": offset, "min_created": min_created, "max_created": max_created}
        url = f"listings/{listing_id}/reviews"
        
        response = self.session.request(url, params=params)
        return Response[Review](**response)
    
    def get_reviews_by_shop_id(self, shop_id: int, limit: int = 25, offset: int = 0, min_created: int = 946684800, max_created: int = None):
        
        params = {"limit": limit, "offset": offset, "min_created": min_created, "max_created": max_created}
        url = f"shops/{shop_id}/reviews"
        
        response = self.session.request(url, params=params)
        return Response[Review](**response)
    
    async def async_get_reviews_by_shop_id(self, session: AsyncOAuth2Session, shop_id: int, limit: int = 25, offset: int = 0):
        
        params = {"limit": limit, "offset": offset}
        url = f"shops/{shop_id}/reviews"

        response = await self.session.async_request(session=session, endpoint=url, params=params)
        return Response[Review](**response)


from __future__ import annotations
from typing import List, Optional
from pydantic import BaseModel

class PropertyValue(BaseModel):
    value_id: Optional[int] = None
    name: str
    scale_id: Optional[int] = None
    equal_to: List[int]
    
class Scale(BaseModel):
    scale_id: int
    display_name: str
    description: str

class ProductProperty(BaseModel):
    """
    Taxonomy Product Properties
    """
    property_id: int
    name: str
    display_name: str
    scales: List[Scale]
    is_required: bool
    supports_attributes: bool
    supports_variations: bool
    is_multivalued: bool
    max_values_allowed: Optional[int]
    possible_values: List[PropertyValue]
    selected_values: List[PropertyValue]

class Taxonomy(BaseModel):
    id: int
    level: int
    name: str
    children: List[Taxonomy]
    full_path_taxonomy_ids: List[int]
    parent_id: Optional[int] = None


from __future__ import annotations
from typing import List
from pydantic import BaseModel
from .PriceModel import Price

class Offering(BaseModel):
    offering_id: int
    quantity: int
    is_enabled: bool
    is_deleted: bool
    price: Price


from pydantic import BaseModel
from typing import List, Optional
from .ShippingProfileUpgradeModel import ShippingProfileUpgrade
from .ShippingProfileDestinationModel import ShippingProfileDestination


class ShippingProfile(BaseModel):
    shipping_profile_id: int
    title: str
    user_id: int
    min_processing_days: Optional[int]
    max_processing_days: Optional[int]
    processing_days_display_label: str
    origin_country_iso: str
    is_deleted: bool
    shipping_profile_destinations: List[ShippingProfileDestination]
    shipping_profile_upgrades: List[ShippingProfileUpgrade]
    origin_postal_code: Optional[str]
    profile_type: str
    domestic_handling_fee: int
    international_handling_fee: int


from __future__ import annotations
from typing import List
from pydantic import BaseModel
from .OfferingModel import Offering
from .PropertyValueModel import PropertyValue

class Product(BaseModel):
    product_id: int
    sku: str
    is_deleted: bool
    offerings: List[Offering]
    property_values: List[PropertyValue]


from __future__ import annotations
from typing import Optional
from pydantic import BaseModel
from .PriceModel import Price

class ShippingProfileDestination(BaseModel):
    shipping_profile_destination_id: int
    shipping_profile_id: int
    origin_country_iso: str
    destination_country_iso: str
    destination_region: str
    primary_cost: Price
    secondary_cost: Price
    shipping_carrier_id: Optional[int]
    mail_class: Optional[str]
    min_delivery_days: Optional[int]
    max_delivery_days: Optional[int]


from dataclasses import dataclass
from typing import List, Optional
from pydantic import BaseModel
import pycountry
from .PriceModel import Price
from .TransactionModel import Transaction
from .ShipmentModel import Shipment
from .RefundModel import Refund





@dataclass(frozen=True)
class ReceiptType:
    UNSHIPPED = {"was_paid":True, "was_shipped":False}
    SHIPPED = {"was_paid":True, "was_shipped":True}
    UNPAID = {"was_paid":False}
    PAID = {"was_paid":True}
    CANCELED = {"was_canceled":True}


@dataclass(frozen=True)
class ReceiptStatus:
    CANCELED = "Canceled"
    PAID = "Paid"
    COMPLETED = "Completed"
    OPEN = "Open"
    PAYMENT_PROCESSING = "Payment Processing"
    

class Receipt(BaseModel):
    receipt_id: int
    receipt_type: int
    seller_user_id: int
    seller_email: str
    buyer_user_id: int
    buyer_email: Optional[str]
    name: str
    first_line: str
    second_line: Optional[str]
    city: Optional[str]
    state: Optional[str]
    zip: Optional[str]
    status: str
    formatted_address: str
    country_iso: str
    payment_method: str
    payment_email: str
    message_from_seller: Optional[str]
    message_from_buyer: Optional[str]
    message_from_payment: Optional[str]
    is_paid: bool
    is_shipped: bool
    create_timestamp: int
    update_timestamp: int
    is_gift: bool
    gift_message: Optional[str]
    grandtotal: Price
    subtotal: Price
    total_price: Price
    total_shipping_cost: Price
    total_tax_cost: Price
    total_vat_cost: Price
    discount_amt: Price
    gift_wrap_price: Price
    shipments: Optional[List[Shipment]]
    transactions: List[Transaction]
    refunds: List[Refund]
    
    @property
    def country_name(self):
        return pycountry.countries.get(alpha_2=self.country_iso).name

    @property
    def is_gift_wrap(self):
        if self.gift_wrap_price.amount > 0:
            return True
        return False


from dataclasses import dataclass

@dataclass
class TokenExpiredError(BaseException):
    status_code: int
    message: str
    error: str
    
    def __repr__(self) -> str:
        return f"""
        \r{30*'-'}
        \rStatus Code: {self.status_code}
        \rError: {self.error}
        \rError Message: {self.message}
    """
    
    def __str__(self) -> str:
        return f"""
        \r{30*'-'}
        \rStatus Code: {self.status_code}
        \rError: {self.error}
        \rError Message: {self.message}
    """


from __future__ import annotations
from pydantic import BaseModel

class File(BaseModel):
    listing_file_id: int
    listing_id: int
    rank: int
    filename: str
    filesize: str
    size_bytes: int
    filetype: str
    create_timestamp: int
    created_timestamp: int
    


from dataclasses import dataclass
from etsy_apiv3.utils import EtsySession, Response
from etsy_apiv3.models import Receipt, ReceiptType, Transaction
from typing import Union
from etsy_apiv3.utils.EtsyOauth2Session import EtsyOauth2Session

@dataclass
class ReceiptResource:
    """
    Receipt Resource is the utility class to get receipts from Etsy
    
    """
    session: Union[EtsySession, EtsyOauth2Session]
    
    def get_receipts(self, shop_id: int, limit=25, offset=0, type: ReceiptType = ReceiptType.UNSHIPPED, **kwargs) -> Response[Receipt]:
        """
        Get Shop Receipts By Shop ID And Receipt Type

        Args:
            
            shop_id (int): Your SHOP ID
            limit (int, optional): Limit Of Receipt Objects. Defaults to 25.
            offset (int, optional): How many receipts to skip. Defaults to 0.
            type (ReceiptType, optional): Receipt type to find. Defaults to ReceiptType.UNSHIPPED.

        Returns:
            Response[Receipt]: Create Response Object Derived from the Receipt Object from json
        """
        
        endpoint = f"shops/{shop_id}/receipts"
        
        params = {"limit": limit, "offset": offset}
        params.update(type)
        params.update(kwargs)
        json = self.session.request(endpoint=endpoint, params=params)
        
        return Response[Receipt](**json)
    
    def get_receipt_by_id(self, shop_id: int, receipt_id: int) -> Receipt:
        """
        Find One Receipt By Shop ID And Receipt ID

        Args:
            shop_id (int): Your SHOP ID
            receipt_id (int): Receipt ID
            
        Returns:
            Receipt: Create Receipt Object from json
        """
        endpoint = f"shops/{shop_id}/receipts/{receipt_id}"
        json = self.session.request(endpoint=endpoint)
        
        return Receipt(**json)

    async def aget_receipt_by_id(self, shop_id: int, receipt_id: int) -> Receipt:
        endpoint = f"shops/{shop_id}/receipts/{receipt_id}"
        json = await self.session.async_request(endpoint)
        return Receipt(**json)
    
    async def aget_receipts(self, shop_id: int, limit: int = 25, offset: int = 0, type: ReceiptType = ReceiptType.UNSHIPPED, **kwargs):
        """
        Get Shop Receipts By Shop ID And Receipt Type

        Args:
            
            shop_id (int): Your SHOP ID
            limit (int, optional): Limit Of Receipt Objects. Defaults to 25.
            offset (int, optional): How many receipts to skip. Defaults to 0.
            type (ReceiptType, optional): Receipt type to find. Defaults to ReceiptType.UNSHIPPED.

        Returns:
            Response[Receipt]: Create Response Object Derived from the Receipt Object from json
        """
        
        endpoint = f"shops/{shop_id}/receipts"
        
        params = {"limit": limit, "offset": offset}
        params.update(type)
        params.update(kwargs)
        json = await self.session.async_request(endpoint=endpoint, params=params)
        
        return Response[Receipt](**json)
    
    def create_shipment(self, shop_id: int, receipt_id: int, tracking_number: str, carrier_name: str, send_bcc: bool = True, note_to_buyer: str = "") -> Receipt:
        """
        Adds tracking information to the receipt object

        Args:
            shop_id (int): SHOP ID
            receipt_id (int): Target Receipt ID
            tracking_number (str): Tracking Number
            carrier_name (str): Carrier Name Ex: UPS
            send_bcc (bool, optional): Send Mail. Defaults to True.
            note_to_buyer (str, optional): Note To Buyer From Seller. Defaults to "".

        Returns:
            Receipt: Receipt Object from json
        """
        endpoint = f"shops/{shop_id}/receipts/{receipt_id}/tracking"
        
        data = {
            "tracking_code":tracking_number, "carrier_name":carrier_name,
            "send_bcc":send_bcc, "note_to_buyer":note_to_buyer
        }
        
        json = self.session.request(endpoint=endpoint, method="POST", data=data)
        return Receipt(**json)
    
    def update_shop_receipt(self, shop_id: int, receipt_id: int, was_shipped: bool = None, was_paid: bool = None) -> Receipt:
        endpoint = f"shops/{shop_id}/receipts/{receipt_id}"
        data = {
            "was_shipped": was_shipped,
            "was_paid": was_paid
        }
        response = self.session.request(endpoint, "PUT", data=data)
        return Receipt(**response)
    
    def get_transactions_by_listing(self, shop_id: int, listing_id: int, limit: int = 25, offset: int = 0) -> Response[Transaction]:
        endpoint = f"shops/{shop_id}/listings/{listing_id}/transactions"
        params = {
            "limit": limit,
            "offset": offset
        }
        response = self.session.request(endpoint, params=params)
        return Response[Transaction](**response)
    
    def get_transactions_by_receipt(self, shop_id: int, receipt_id: int) -> Response[Transaction]:
        endpoint = f"shops/{shop_id}/receipts/{receipt_id}/transactions"
        response = self.session.request(endpoint)
        return Response[Transaction](**response)
    
    def get_transaction_by_transaction_id(self, shop_id: int, transaction_id: int) -> Transaction:
        endpoint = f"shops/{shop_id}/transactions/{transaction_id}"
        response = self.session.request(endpoint)
        return Transaction(**response)
    
    def get_receipt_transactions_by_shop(self, shop_id: int, limit: int = 25, offset: int = 0) -> Response[Transaction]:
        endpoint = f"shops/{shop_id}/transactions"
        params = {
            "limit": limit,
            "offset": offset
        }
        response = self.session.request(endpoint, params=params)
        return Response[Transaction](**response)



from etsy_apiv3.models import Receipt, Transaction
from etsy_apiv3.utils import EtsySession, Response
from etsy_apiv3.resources import ListingResource
from etsy_apiv3.utils.RequestException import EtsyRequestException
import test_credentials

session = EtsySession(test_credentials.CLIENT_KEY, test_credentials.CLIENT_SECRET, test_credentials.TOKEN)

resource = ListingResource(session)

for taxonomy in resource.get_buyer_taxonomy_nodes().results:
    a = resource.get_properties_by_buyer_taxonomy_id(taxonomy.id)
    print(a)


CLIENT_KEY = "paiimgc9v4jeh24la7eouzu5"
CLIENT_SECRET = "a2xer79vbd"
TOKEN = {
    "token_type": "Bearer",
    "expires_in": 3600,
    "access_token": "230829585.ehkmCEa8V-RRcs86iWjjQaR-5Cmd_vbRT9S-sVBkwd-QnKqkQA3POgANXNKe0YyfIM5Sz-gWH44gU4FM4x1v8mYwQR",
    "refresh_token": "230829585.cW0xCY0nXLsBXc3UdH9CseCCT4P5j0yk6XOUd9nfz1lMRD6ycU9XIcJQL9UwhRoyq1mW-YC_09koNZDzjR-p7VMK5Q",
    "expires_at": 1673517018.2133296
}


from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel
class PropertyValue(BaseModel):
    property_id: int
    property_name: Optional[str]
    scale_id: Optional[int]
    scale_name: Optional[str]
    value_ids: List[int]
    values: List[str]


from .ListingImageModel import ListingImage
from .ListingModel import Listing, DraftListing, VariationImage, Video
from .ProductionPartnerModel import ProductionPartner
from .ShippingProfileModel import ShippingProfile
from .ShopModel import Shop
from .TranslationModel import Translation
from .UserModel import User, Me
from .PriceModel import Price
from .ProductModel import Product
from .ReceiptModel import Receipt, ReceiptStatus, ReceiptType
from .OfferingModel import Offering
from .PropertyValueModel import PropertyValue
from .ReviewModel import Review
from .ShippingProfileDestinationModel import ShippingProfileDestination
from .ShipmentModel import Shipment
from .TransactionModel import Transaction
from .VariationModel import Variation
from .ShippingProfileUpgradeModel import ShippingProfileUpgrade
from .RefundModel import Refund
from .ListingModel import Inventory
from .ListingPropertyModel import ListingProperty
from .File import File
from .ShopSection import ShopSection
from .ReturnPolicyModel import ReturnPolicy
from .TaxonomyModel import Taxonomy, ProductProperty
from .ShippingCarrierModel import ShippingCarrier
from .PaymentModel import Payment, PaymentAccountLedgerEntry


from __future__ import annotations
import html
from typing import Annotated, List, Optional
from pydantic import BaseModel, validator

from .ListingImageModel import ListingImage 
from .ProductionPartnerModel import ProductionPartner
from .ShippingProfileModel import ShippingProfile
from .ShopModel import Shop
from .TranslationModel import Translation
from .UserModel import User
from .PriceModel import Price
from .ProductModel import Product

class Video(BaseModel):
    video_id: int
    height: int
    width: int
    thumbnail_url: str
    video_url: str
    video_state: str
    

class VariationImage(BaseModel):
    property_id: int
    value_id: int
    value: Optional[str] = None
    image_id: int

class Inventory(BaseModel):
    products: List[Product]
    price_on_property: List[int]
    quantity_on_property: List[int]
    sku_on_property: List[int]
    listing: Optional[Listing]


class DraftListing(BaseModel):
    quantity: int
    title: str
    description: str
    price: float
    who_made: str = "i_did"
    when_made: str = "made_to_order"
    taxonomy_id: int
    shipping_profile_id: Optional[int]
    return_policy_id: Optional[int]
    materials: Optional[List[str]]
    shop_section_id: Optional[int]
    processing_min: Optional[int]
    processing_max: Optional[int]
    tags: Optional[List[str]]
    styles: Optional[List[str]]
    item_weight: Optional[float]
    item_length: Optional[float]
    item_width: Optional[float]
    item_height: Optional[float]
    item_weight_unit: Optional[str]
    item_dimensions_unit: Optional[str]
    is_personalizable: bool
    personalization_is_required: Optional[bool]
    personalization_char_count_max: Optional[int]
    personalization_instructions: Optional[int]
    production_partner_ids: Optional[List[int]]
    image_ids: Optional[List[int]]
    is_supply: bool
    is_customizable: bool
    should_auto_renew: bool
    is_taxable: bool
    type: str = "physical"
    
class Listing(BaseModel):
    listing_id: Annotated[int, "listing_id"]
    user_id: int
    shop_id: int
    title: str
    description: str
    state: str
    creation_timestamp: int
    ending_timestamp: int
    original_creation_timestamp: int
    last_modified_timestamp: int
    state_timestamp: int
    quantity: int
    shop_section_id: Optional[int]
    featured_rank: int
    url: str
    num_favorers: int
    non_taxable: bool
    is_customizable: bool
    is_personalizable: bool
    personalization_is_required: bool
    personalization_char_count_max: Optional[int]
    personalization_instructions: Optional[str]
    listing_type: str
    tags: List[str]
    materials: List[str]
    shipping_profile_id: Optional[int]
    processing_min: Optional[int]
    processing_max: Optional[int]
    who_made: Optional[str]
    when_made: Optional[str]
    is_supply: bool
    item_weight: Optional[int]
    item_weight_unit: Optional[str]
    item_length: Optional[int]
    item_width: Optional[int]
    item_height: Optional[int]
    item_dimensions_unit: Optional[str]
    is_private: bool
    style: List[str]
    file_data: str
    has_variations: bool
    should_auto_renew: bool
    language: str
    price: Price
    taxonomy_id: int
    shipping_profile: Optional[ShippingProfile]
    user: Optional[User]
    shop: Optional[Shop]
    images: Optional[List[ListingImage]]
    production_partners: List[ProductionPartner]
    skus: List[str]
    translations: Optional[List[Translation]]
    inventory: Optional[Inventory]
    videos: Optional[List[Video]]
    views: Optional[int]

    @validator("title")
    def title_validator(cls, value):
        new_value = html.unescape(value)
        return new_value
    
    @validator("description")
    def description_validator(cls, value):
        new_value = html.unescape(value)
        return new_value
    

Inventory.update_forward_refs()



from typing import Coroutine
import unittest
from etsy_apiv3.models import Receipt, Transaction
from etsy_apiv3.utils import EtsySession, Response
from etsy_apiv3.resources import ReceiptResource
from etsy_apiv3.utils.RequestException import EtsyRequestException
import test_credentials

class TestReceipt(unittest.TestCase):
    
    SUCCESS_RECEIPT_ID = "2672482884"
    WRONG_RECEIPT_ID = "2672482222"
    SUCCESS_TRANSACTION_ID = "3246274562"
    WRONG_TRANSACTION_ID = "3246274561"
    SUCCESS_LISTING_ID = "963147161"
    WRONG_LISTING_ID = "963147162"
    WRONG_SHOP_ID = "3213213"
    
    def setUp(self) -> None:
        self.session = EtsySession(
            test_credentials.CLIENT_KEY, test_credentials.CLIENT_SECRET,
            test_credentials.TOKEN
        )
        self.resource = ReceiptResource(self.session)
        self.longMessage = True
        
    def test_get_receipt_by_id(self):
        success_receipt = self.resource.get_receipt_by_id(self.session.me.shop_id, self.SUCCESS_RECEIPT_ID)
        async_success_receipt = self.resource.aget_receipt_by_id(self.session.me.shop_id, self.SUCCESS_RECEIPT_ID)
        self.assertIsInstance(
            async_success_receipt, Coroutine
        )
        
        print(async_success_receipt)
        self.assertIsInstance(
            success_receipt, Receipt
        )
        
        with self.assertRaises(EtsyRequestException):
            self.resource.get_receipt_by_id(self.session.me.shop_id, self.WRONG_RECEIPT_ID)
    
    def test_get_receipts(self):
        receipts = self.resource.get_receipts(self.session.me.shop_id)
        self.assertIsInstance(
            receipts, Response
        )
        with self.assertRaises(EtsyRequestException):
            self.resource.get_receipts("321321321")
    
    def test_get_transaction_by_transaction_id(self):
        
        """
        Get Transaction By Success Transaction ID
        If transaction isinstance :class:`etsy_apiv3.models.Transaction`] return True
        Else Assert
        
        If Wrong Transaction ID Return EtsyRequestException return True
        Else AssertionError
        """
        
        transaction = self.resource.get_transaction_by_transaction_id(self.session.me.shop_id, self.SUCCESS_TRANSACTION_ID)
        self.assertIsInstance(transaction, Transaction, "Transaction Get Successfully By Transaction ID")
        with self.assertRaises(EtsyRequestException):
            self.resource.get_transaction_by_transaction_id(self.session.me.shop_id, self.WRONG_TRANSACTION_ID)
    
    def test_get_transactions_by_receipt_id(self):
        transactions = self.resource.get_transactions_by_receipt(self.session.me.shop_id, self.SUCCESS_RECEIPT_ID)
        self.assertIsInstance(
            transactions, Response
        )
        """with self.assertRaises(EtsyRequestException):
            self.resource.get_transactions_by_receipt(self.session.me.shop_id, self.WRONG_RECEIPT_ID)
        """
            
    def test_get_transactions_by_listing_id(self):
        transactions = self.resource.get_transactions_by_listing(self.session.me.shop_id, self.SUCCESS_LISTING_ID, 25)
        self.assertIsInstance(
            transactions, Response[Transaction]
        )
        with self.assertRaises(EtsyRequestException):
            self.resource.get_transactions_by_listing(self.session.me.shop_id, self.WRONG_LISTING_ID)
    
    def test_get_transactions_by_shop_id(self):
        transactions = self.resource.get_receipt_transactions_by_shop(self.session.me.shop_id)
        self.assertIsInstance(
            transactions, Response
        )
        with self.assertRaises(EtsyRequestException):
            self.resource.get_receipt_transactions_by_shop(self.WRONG_SHOP_ID)
        
    
unittest.main()
        


from typing import Optional
from etsy_apiv3.utils import EtsySession, Response, EtsyOauth2Session
from dataclasses import dataclass
from etsy_apiv3.models import ReturnPolicy
from typing import Union

@dataclass
class ShopPolicyResource:
    """
    Shop Policy Resource Of Etsy Api V3.

    """
    session: Union[EtsySession, EtsyOauth2Session]
    
    def get_shop_return_policies(self, shop_id: int) -> Response[ReturnPolicy]:
        endpoint = f"shops/{shop_id}/policies/return"
        response = self.session.request(endpoint)
        return Response[ReturnPolicy](**response)
    
    def get_shop_return_policy_by_id(self, shop_id: int, return_policy_id: int) -> ReturnPolicy:
        endpoint = f"shops/{shop_id}/policies/return/{return_policy_id}"
        response = self.session.request(endpoint)
        return ReturnPolicy(**response)
    
    def create_shop_return_policy(self, shop_id: int, accepts_returns: bool, accepts_exchanges: bool, return_deadline: Optional[int]) -> ReturnPolicy:
        endpoint = f"shops/{shop_id}/policies/return"
        data = {
            "accepts_returns": accepts_returns,
            "accepts_exchanges": accepts_exchanges
        }
        if return_deadline:
            data.update({"return_deadline": return_deadline})
        
        response = self.session.request(endpoint, "POST", data=data)
        return ReturnPolicy(**response)
    
    def update_shop_return_policy(self, shop_id: int, return_policy_id: int, accepts_returns: bool, accepts_exchanges: bool, return_deadline: Optional[int] = None) -> ReturnPolicy:
        endpoint = f"shops/{shop_id}/policies/return/{return_policy_id}"
        data = {
            "accepts_returns": accepts_returns,
            "accepts_exchanges": accepts_exchanges
        }
        if return_deadline:
            data.update({"return_deadline": return_deadline})
        
        response = self.session.request(endpoint, "PUT", data=data)
        return ReturnPolicy(**response)
    
    def delete_shop_return_policy(self, shop_id: int, return_policy_id: int) -> str:
        endpoint = f"shops/{shop_id}/policies/return/{return_policy_id}"
        response = self.session.request(endpoint, "DELETE")
        return response
    
    def consolidate_shop_return_policies(self, shop_id: int, source_return_policy_id: int, destination_return_policy_id: int) -> ReturnPolicy:
        endpoint = f"shops/{shop_id}/policies/return/consolidate"
        data = {
            "source_return_policy_id": source_return_policy_id,
            "destination_return_policy_id": destination_return_policy_id
        }
        response = self.session.request(endpoint, "POST", data=data)
        return ReturnPolicy(**response)
    
    


from dataclasses import dataclass
from typing import List, Optional, Union
from etsy_apiv3.models import ProductionPartner, Shop, ShopSection
from etsy_apiv3.utils import EtsySession, Response, EtsyOauth2Session

@dataclass
class ShopResource:
    """
    Shop Resource Of Etsy Api V3.

    """
    session: Union[EtsySession, EtsyOauth2Session]
    
    def get_shop_by_id(self, shop_id: int):
        """
        Get Shop By Shop Id And Return A Shop Object

        Args:
            shop_id (int): Shop Id

        Returns:
            Shop: Shop Object
        """
        
        endpoint = f"shops/{shop_id}"
        response = self.session.request(endpoint)
        return Shop(**response)
    
    def find_shops(self, shop_name: str, limit: int = 25, offset: int = 0):
        """
        Find All Shops And Return Response Object
        
        Returns:
            Response: Return Response object of shop list 
        """
        endpoint = "shops"
        response = self.session.request(endpoint, params={"shop_name":shop_name, "limit": limit, "offset": offset})
        return Response[Shop](**response)

    def get_shop_by_owner_user_id(self, user_id: int):
        """
        Find Shop By Owner User Id And Return A Shop Object.

        Args:
            user_id (int): Shop Owner User Id

        Returns:
            Shop: Return A Shop Object
        """
        endpoint = f"users/{user_id}/shops"
        response = self.session.request(endpoint)
        return Shop(**response)
    
    def update_shop(self, shop_id: int, title: Optional[str] = None, announcement: Optional[str] = None, sale_message: Optional[str] = None, digital_sale_message: Optional[str] = None) -> Shop:
        data = {}
        if title:
            data.update({"title": title})
        if announcement:
            data.update({"announcement": announcement})
        if sale_message:
            data.update({"sale_message": sale_message})
        if digital_sale_message:
            data.update({"digital_sale_message": digital_sale_message})
            
        endpoint = f"shops/{shop_id}"
        response = self.session.request(endpoint, "PUT", data=data)
        return Shop(**response)
    
    def get_shop_production_partners(self, shop_id: int) -> List[ProductionPartner]:
        endpoint = f"shops/{shop_id}/production-partners"
        response = self.session.request(endpoint)
        return Response[ProductionPartner](**response)
    
    def get_shop_sections(self, shop_id: int) -> List[ShopSection]:
        endpoint = f"shops/{shop_id}/sections"
        response = self.session.request(endpoint)
        return Response[ShopSection](**response)
    
    def get_shop_section_by_id(self, shop_id: int, shop_section_id: int) -> ShopSection:
        endpoint = f"shops/{shop_id}/sections/{shop_section_id}"
        response = self.session.request(endpoint)
        return ShopSection(**response)
    
    def delete_shop_scetion_by_id(self, shop_id: int, shop_section_id: int) -> str:
        endpoint = f"shops/{shop_id}/sections/{shop_section_id}"
        response = self.session.request(endpoint, "DELETE")
        return response
    
    def create_shop_section(self, shop_id: int, title: str) -> ShopSection:
        endpoint = f"shops/{shop_id}/sections"
        response = self.session.request(endpoint, "POST", data={"title": title})
        return ShopSection(**response)
    
    def update_shop_section(self, shop_id: int, shop_section_id: int, title: str) -> ShopSection:
        endpoint = f"shops/{shop_id}/sections/{shop_section_id}"
        response = self.session.request(endpoint, "PUT", data={"title": title})
        return ShopSection(**response)
    
    
    


from pydantic import BaseModel

class ShippingCarrierClasses(BaseModel):
    mail_class_key: str
    name: str

class ShippingCarrier(BaseModel):
    shipping_carrier_id: int
    name: str
    domestic_classes: ShippingCarrierClasses
    international_classes: ShippingCarrierClasses


from dataclasses import dataclass
from typing import Callable, List, Optional
from httpx import Client, URL
from etsy_apiv3.models.TokenModel import Token
from etsy_apiv3.exceptions.TokenExpiredError import TokenExpiredError
from etsy_apiv3.utils.RequestException import EtsyRequestException
from etsy_apiv3.utils.EtsyListRequestException import EtsyListRequestException
from etsy_apiv3.models import Me

AUTH_CONNECT_ENDPOINT = "https://www.etsy.com/oauth/connect"
TOKEN_ENDPOINT = "https://api.etsy.com/v3/public/oauth/token"


@dataclass
class APILimit:
    limit_per_second: Optional[int]
    remaining_this_second: Optional[int]
    limit_per_day: Optional[int]
    remaining_today: Optional[int]


class EtsyOauth2Session:
    
    def __init__(self, client_id: str, token: Optional[Token] = None, token_updater: Optional[Callable] = None, extra_token_updater_args: Optional[tuple] = None) -> 'EtsyOauth2Session':
        self.headers = {
            "x-api-key": client_id
        }
        self.token = token
        
        if self.token is not None:
            if token_updater is None:
                self.token_updater = self._token_updater
            else:
                self.token_updater = token_updater
                
            self.headers["Authorization"] = f"{token.token_type} {token.access_token}"
        self.client_id = client_id
        self.client = Client(timeout=30.0)
        self.limits = APILimit(
            limit_per_second=None,
            remaining_this_second=None,
            limit_per_day=None,
            remaining_today=None
        )
        
        self.__me = None
        self.extra_token_updater_args = extra_token_updater_args
        
    @property
    def me(self) -> Me:
        if self.__me is None:
            self.__me = Me(**self.request(f"users/me"))
        return self.__me
    
    def _token_updater(self, new_token: Token, *args):
        self.token = new_token
        return True
    
    def refresh_token(self):
        data = {
            "grant_type": "refresh_token",
            "client_id": self.client_id,
            "refresh_token": self.token.refresh_token
        }
        response = self.client.post(TOKEN_ENDPOINT, data=data)
        token = Token(**response.json())
        self.headers["Authorization"] = f"{token.token_type} {token.access_token}"
        
        if self.extra_token_updater_args is not None:
            
            self.token_updater(token, *self.extra_token_updater_args)
        else:
            self.token_updater(token)
        
        return True
    
    def request(self, endpoint: str, method: str ="GET", *args, **kwargs) -> dict:
        """ method
        Send Request to target endpoint by method

        Args:
            endpoint (str): Api Endpoint Url
            method (str, optional): HTTP Methods [GET, POST, PUT, DELETE, UPDATE]. Defaults to "GET".

        Raises:
            EtsyRequestException: EtsyRequestException(status_code, message)

        Returns:
            json: Json From Request Response
        """
        base_endpoint = "https://openapi.etsy.com/v3/application/"
        url = f"{base_endpoint}{endpoint}"
        
        headers = self.headers
        if kwargs.get("headers") is not None:
            headers.update(kwargs.get("headers"))
            kwargs.pop("headers")
        
        res = self.client.request(method, url, headers=headers, *args, **kwargs)
        res_json = res.json()
        
        if "X-Limit-Per-Day" in res.headers.keys():
            
            self.limits.limit_per_day = res.headers["X-Limit-Per-Day"]
            self.limits.limit_per_second = res.headers["X-Limit-Per-Second"]
            self.limits.remaining_this_second = res.headers["X-Remaining-This-Second"]
            self.limits.remaining_today = res.headers["X-Remaining-Today"]

        
        if res.status_code in [400, 404, 500]:
            
            if isinstance(res_json, list):
                raise EtsyListRequestException(res.status_code, errors=res_json)
            
            raise EtsyRequestException(res.status_code, res_json["error"])
        
        if res.status_code == 401:
            if res_json["error"] == "invalid_token":
                updated = self.refresh_token()
                if updated:
                    return self.request(endpoint, method, *args, **kwargs)
                else:
                    raise TokenExpiredError(
                        status_code=res.status_code,
                        message=res_json["error_description"],
                        error=res_json["error"]
                    )
            
            raise EtsyRequestException(res.status_code, res_json["error"])
        
        
        return res_json
            
        


class EtsyOauth2Helper:
    
    def authorize(self, client_id: str, redirect_uri: str, scopes: List[str], state: str = "superuser", code_challange: str = None) -> str:
        authorize_url = URL(AUTH_CONNECT_ENDPOINT, params={
            "response_type": "code",
            "client_id": client_id,
            "redirect_uri": redirect_uri,
            "scope": " ".join(scopes),
            "code_challenge": code_challange,
            "state": state,
            "code_challenge_method": "S256"
        })
        print(f"Go to the following URL: {authorize_url}")
        return f"{authorize_url.scheme}://{authorize_url.host}{authorize_url.path}?{authorize_url.query.decode('utf-8')}"
    
    def authenticate(self, auth_code: str, redirect_uri: str, client_id: str, code_verifier: str) -> EtsyOauth2Session:
        client = Client(http2=False)
        data = {
            "grant_type": "authorization_code",
            "code": auth_code,
            "redirect_uri": redirect_uri,
            "client_id": client_id,
            "code_verifier": code_verifier
        }
        response = client.post(TOKEN_ENDPOINT, data=data)
        token = response.json()
        return EtsyOauth2Session(client_id, token=Token(**token))
    



"""
Etsy Api V3 Sdk PaymentResource Tests.
"""
import unittest
from etsy_apiv3.models import Payment
from etsy_apiv3.utils import EtsySession, Response, EtsyRequestException
from etsy_apiv3.resources import PaymentResource

import test_credentials

class TestTransaction(unittest.TestCase):
    SUCCESS_RECEIPT_ID = "2672482884"
    WRONG_RECEIPT_ID = "2672482222"
    PAYMENT_IDS = [119519091349]
    WRONG_PAYMENT_IDS = [119519091322]
    LEDGER_ENTRY_IDS = []
    WRONG_LEDGER_ENTRY_IDS = []
    
    def setUp(self) -> None:
        self.session = EtsySession(
            test_credentials.CLIENT_KEY, test_credentials.CLIENT_SECRET,
            test_credentials.TOKEN
        )
        self.resource = PaymentResource(self.session)
        self.longMessage = True
    
    def test_get_payment_by_receipt_id(self):
        payment = self.resource.get_payment_by_receipt_id(self.session.me.shop_id, self.SUCCESS_RECEIPT_ID)
        self.assertIsInstance(
            payment, Response[Payment]
        )
        with self.assertRaises(EtsyRequestException):
            self.resource.get_payment_by_receipt_id(self.session.me.shop_id, self.WRONG_RECEIPT_ID)
    
    def test_get_shop_payments(self):
        payments = self.resource.get_shop_payments(self.session.me.shop_id, self.PAYMENT_IDS)
        self.assertIsInstance(
            payments, Response[Payment]
        )
        with self.assertRaises(EtsyRequestException):
            self.resource.get_payment_by_receipt_id(self.session.me.shop_id, self.WRONG_PAYMENT_IDS)

    
    def test_get_payment_by_account_ledger_entry_ids(self):
        payment = self.resource.get_payment_by_account_ledger_entry_ids(self.session.me.shop_id, self.LEDGER_ENTRY_IDS)
        self.assertIsInstance(
            payment, Response[Payment]
        )
        with self.assertRaises(EtsyRequestException):
            self.resource.get_payment_by_account_ledger_entry_ids(
                self.session.me.shop_id,
                self.WRONG_LEDGER_ENTRY_IDS
            )
    
    def test_get_payment__(self):
        self.resource.get_shop_payment_account_ledger_entries(self.session.me.shop_id, )
        pass
    
    
unittest.main()


from __future__ import annotations
from pydantic import BaseModel
from typing import List, Optional
from .PriceModel import Price
from .VariationModel import Variation

class Transaction(BaseModel):
    transaction_id: int
    title: str
    description: str
    seller_user_id: int
    buyer_user_id: int
    create_timestamp: int
    paid_timestamp: Optional[int]
    shipped_timestamp: Optional[int]
    quantity: int
    listing_image_id: int
    receipt_id: int
    is_digital: bool
    file_data: str
    listing_id: int
    transaction_type: str
    product_id: int
    sku: str
    price: Price
    shipping_cost: Price
    variations: List[Variation]
    shipping_profile_id: Optional[int]
    min_processing_days: Optional[int]
    max_processing_days: Optional[int]
    shipping_method: Optional[str]
    shipping_upgrade: Optional[str]
    expected_ship_date: Optional[int]
    buyer_coupon: float
    shop_coupon: float
    



from typing import Optional, List
from pydantic import BaseModel
from .PriceModel import Price



class PaymentAdjustmentItem(BaseModel):
    payment_adjustment_id: int
    payment_adjustment_item_id: int
    adjustment_type: Optional[str]
    amount: int = 0
    shop_amount: int = 0
    transaction_id: Optional[int]
    bill_payment_id: Optional[int]
    created_timestamp: int
    updated_timestamp: int

class PaymentAdjustment(BaseModel):
    payment_adjustment_id: int
    payment_id: int
    status: str
    is_success: bool
    user_id: int
    reason_code: str
    total_adjustment_amount: Optional[int]
    shop_total_adjustment_amount: Optional[int]
    buyer_total_adjustment_amount: Optional[int]
    total_fee_adjustment_amount: Optional[int]
    create_timestamp: int
    created_timestamp: int
    update_timestamp: int
    updated_timestamp: int
    payment_adjustment_items: List[PaymentAdjustmentItem]


class PaymentAccountLedgerEntry(BaseModel):
    entry_id: int
    ledger_id: int
    sequence_number: int
    amount: int
    currency: str
    description: str
    balance: int
    create_date: int
    created_timestamp: int
    ledger_type: str
    reference_type: str
    reference_id: Optional[str]
    payment_adjustments: List[PaymentAdjustment]

class Payment(BaseModel):
    payment_id: int
    buyer_user_id: int
    shop_id: int
    receipt_id: int
    amount_gross: Price
    amount_fees: Price
    amount_net: Price
    posted_gross: Price
    posted_fees: Price
    posted_net: Price
    adjusted_gross: Optional[Price]
    adjusted_fees: Optional[Price]
    adjusted_net: Optional[Price]
    currency: str
    shop_currency: Optional[str]
    shipping_user_id: Optional[int]
    shipping_address_id: Optional[int]
    billing_address_id: Optional[int]
    status: str
    shipped_timestamp: Optional[int]
    create_timestamp: Optional[int]
    created_timestamp: Optional[int]
    update_timestamp: Optional[int]
    updated_timestamp: Optional[int]
    payment_adjustments: List[PaymentAdjustment]
    
    


from dataclasses import dataclass
from typing import List, Optional, Union
from etsy_apiv3.utils import EtsySession, Response, EtsyOauth2Session
from etsy_apiv3.models import ShippingCarrier, ShippingProfile, ShippingProfileDestination, ShippingProfileUpgrade

@dataclass
class ShippingResource:
    """
    Shop Resource Of Etsy Api V3.

    """
    session: Union[EtsySession, EtsyOauth2Session]
    
    def get_shipping_carriers(self, origin_country_iso: str) -> Response[ShippingCarrier]:
        endpoint = "shipping-carriers"
        response = self.session.request(endpoint, params={"origin_country_iso": origin_country_iso})
        return Response[ShippingCarrier](**response)
    
    def get_shop_shipping_profiles(self, shop_id: int) -> Response[ShippingProfile]:
        endpoint = f"shops/{shop_id}/shipping-profiles"
        response = self.session.request(endpoint)
        return Response[ShippingProfile](**response)
    
    def get_shop_shipping_profile(self, shop_id: int, shipping_profile_id: int) -> ShippingProfile:
        endpoint = f"shops/{shop_id}/shipping-profiles/{shipping_profile_id}"
        response = self.session.request(endpoint)
        return ShippingProfile(**response)
    
    def get_shop_shipping_profile_destinations(self, shop_id: int, shipping_profile_id: int, limit: int = 25, offset: int = 0) -> Response[ShippingProfileDestination]:
        endpoint = f"shops/{shop_id}/shipping-profiles/{shipping_profile_id}/destinations"
        params = {
            "limit": limit,
            "offset": offset
        }
        response = self.session.request(endpoint, params=params)
        return Response[ShippingProfileDestination](**response)
    
    def get_shop_shipping_profile_upgrades(self, shop_id: int, shipping_profile_id: int) -> Response[ShippingProfileUpgrade]:
        endpoint = f"shops/{shop_id}/shipping-profiles/{shipping_profile_id}/upgrades"
        response = self.session.request(endpoint)
        return Response[ShippingProfileUpgrade](**response)
    
    def delete_shop_shipping_profile(self, shop_id: int, shipping_profile_id: int) -> str:
        endpoint = f"shops/{shop_id}/shipping-profiles/{shipping_profile_id}"
        response = self.session.request(endpoint, "DELETE")
        return response
    
    def delete_shop_shipping_profile_destination(self, shop_id: int, shipping_profile_id: int, shipping_profile_destination_id: int) -> str:
        endpoint = f"shops/{shop_id}/shipping-profiles/{shipping_profile_id}/destinations/{shipping_profile_destination_id}"
        response = self.session.request(endpoint, "DELETE")
        return response
    
    def delete_shop_shipping_profile_upgrade(self, shop_id: int, shipping_profile_id: int, upgrade_id: int) -> str:
        endpoint = f"shops/{shop_id}/shipping-profiles/{shipping_profile_id}/upgrades/{upgrade_id}"
        response = self.session.request(endpoint, "DELETE")
        return response
    
    def update_shop_shipping_profile(self, shop_id: int, shipping_profile_id: int, title: str, origin_country_iso: str, min_processing_time: int, max_processing_time: int, processing_time_unit: str = "business_days", origin_postal_code: str = None) -> ShippingProfile:
        endpoint = f"shops/{shop_id}/shipping-profiles/{shipping_profile_id}"
        data = {
            "title": title,
            "origin_country_iso": origin_country_iso,
            "min_processing_time": min_processing_time,
            "max_processing_time": max_processing_time,
            "processing_time_unit": processing_time_unit
        }
        if origin_postal_code:
            data.update({"origin_postal_code": origin_postal_code})
        
        response = self.session.request(endpoint, "PUT", data=data)
        return ShippingProfile(**response)
    
    def update_shop_shipping_profile_destination(self, shop_id: int, shipping_profile_id: int, shipping_profile_destination_id: int, primary_cost: Optional[float] = None, secondary_cost: Optional[float] = None, destination_country_iso: Optional[str] = None, destination_region: Optional[str] = "none", shipping_carrier_id: Optional[int] = None, mail_class: Optional[str] = None, min_delivery_days: Optional[int] = None, max_delivery_days: Optional[int] = None) -> ShippingProfileDestination:
        endpoint = f"shops/{shop_id}/shipping-profiles/{shipping_profile_id}/destinations/{shipping_profile_destination_id}"
        data = {
            "primary_cost": primary_cost,
            "secondary_cost": secondary_cost,
            "destination_country_iso": destination_country_iso,
            "destination_region": destination_region,
            "shipping_carrier_id": shipping_carrier_id,
            "mail_class": mail_class,
            "min_delivery_days": min_delivery_days,
            "max_delivery_days": max_delivery_days
        }
        response = self.session.request(endpoint, "PUT", data=data)
        return ShippingProfileDestination(**response)

    def update_shop_shipping_profile_upgrade(self, shop_id: int, shipping_profile_id: int, upgrade_id: int, upgrade_name: Optional[str] = None, type: str = "0", price: Optional[float] = None, secondary_price: Optional[float] = None, shipping_carrier_id: Optional[int] = None, mail_class: Optional[str] = None, min_delivery_days: Optional[int] = None, max_delivery_days: Optional[int] = None) -> ShippingProfileUpgrade:
        endpoint = f"shops/{shop_id}/shipping-profiles/{shipping_profile_id}/upgrades/{upgrade_id}"
        data = {
            "upgrade_name": upgrade_name,
            "type": type,
            "price": price,
            "secondary_price": secondary_price,
            "shipping_carrier_id": shipping_carrier_id,
            "mail_class": mail_class,
            "min_delivery_days": min_delivery_days,
            "max_delivery_days": max_delivery_days
        }
        response = self.session.request(endpoint, "PUT", data=data)
        return ShippingProfileUpgrade(**response)
    
    
        


from __future__ import annotations
from pydantic import BaseModel
from typing import List, Optional

class Shop(BaseModel):
    shop_id: int
    user_id: int
    shop_name: str
    create_date: int
    title: Optional[str]
    announcement: Optional[str]
    currency_code: str
    is_vacation: bool
    vacation_message: Optional[str]
    sale_message: Optional[str]
    digital_sale_message: Optional[str]
    update_date: int
    listing_active_count: int
    digital_listing_count: int
    login_name: str
    accepts_custom_requests: bool
    policy_welcome: Optional[str]
    policy_payment: Optional[str]
    policy_shipping: Optional[str]
    policy_refunds: Optional[str]
    policy_additional: Optional[str]
    policy_seller_info: Optional[str]
    policy_update_date: int
    policy_has_private_receipt_info: bool
    has_unstructured_policies: bool
    policy_privacy: Optional[str]
    vacation_autoreply: Optional[str]
    url: str
    image_url_760x100: Optional[str]
    num_favorers: int
    languages: List[str]
    icon_url_fullxfull: Optional[str]
    is_using_structured_policies: bool
    has_onboarded_structured_policies: bool
    include_dispute_form_link: bool
    is_direct_checkout_onboarded: bool
    is_etsy_payments_onboarded: bool
    is_calculated_eligible: bool
    is_opted_in_to_buyer_promise: bool
    is_shop_us_based: bool
    transaction_sold_count: int
    shipping_from_country_iso: Optional[str]
    shop_location_country_iso: Optional[str]
    review_count: Optional[int]
    review_average: Optional[float]
    



import setuptools
from setuptools import find_packages

from etsy_apiv3 import __version__

setuptools.setup(
    name="etsy-apiv3",
    version=__version__,
    author="Esat Yılmaz",
    author_email="esatyilmaz3500@gmail.com",
    description="Etsy APIV3 SDK",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
    install_requires=[
        "Async_OAuthlib>=0.0.9",
        "pycountry>=22.1.10",
        "pydantic>=1.10.2",
        "requests>=2.27.1",
        "requests_oauthlib>=1.3.0",
        "setuptools>=58.1.0",
        
    ]
)



from dataclasses import dataclass
from typing import List, Optional, Union
from requests.models import RequestEncodingMixin
from etsy_apiv3.utils import EtsySession, Response
from etsy_apiv3.models import (
    Translation, Product, Offering, Listing,
    Inventory, ListingProperty, File, ListingImage,
    DraftListing, Taxonomy, ProductProperty, VariationImage,
    Video
)
from async_oauthlib import OAuth2Session as AsyncOauth2Session

from etsy_apiv3.utils.EtsyOauth2Session import EtsyOauth2Session


@dataclass
class ListingResource:
    
    session: Union[EtsySession, EtsyOauth2Session]
    
    def get_listing(self, listing_id: int, includes: str = "") -> Listing:
        """
        Retrieves a listing record by listing ID.

        Args:
            listing_id (int): Listing Id
            includes (str, optional): An enumerated string that attaches a valid association. Acceptable inputs are 'Shipping', 'Shop', 'Images', 'User', 'Translations' and 'Inventory'. Defaults to "".

        Returns:
            Listing: A Single Listing Object
        """
        endpoint = f"listings/{listing_id}"
        json = self.session.request(endpoint, params={"includes":includes})
        return Listing(**json)

    def find_all_active_listings(self, limit=25, offset=0, keywords="", sort_on="created", sort_order="desc", min_price=None, max_price=None, taxonomy_id=None, shop_location="United States", _as="pydantic"):
        """
        Find All Active Listings By Keywords, Min price, Max Price Or Shop Location

        Args:
            limit (int, optional): Result Limit. Defaults to 25, Max 100.
            offset (int, optional): Result Offset. Defaults to 0.
            keywords (str, optional): Keywords For Active Listings. Defaults to "".
            sort_on (str, optional): Sort By Result. Defaults to "created".
            sort_order (str, optional): Sort By Order. Defaults to "desc".
            min_price (_type_, optional): Min Price. Defaults to None.
            max_price (_type_, optional): Max Price. Defaults to None.
            taxonomy_id (_type_, optional): Taxonomy Id. Defaults to None.
            shop_location (_type_, optional): Shop Location. Defaults to US.

        Returns:
            Response[Listing]: List of Listing Items
        """
        
        endpoint = f"listings/active"
        params = {
            "limit":limit, "offset":offset, "keywords":keywords, "sort_on":sort_on,
            "sort_order":sort_order, "min_price":min_price, "max_price":max_price,
            "taxonomy_id":taxonomy_id, "shop_location":shop_location
        }
        
        json = self.session.request(endpoint, params=params)
        
        return Response[Listing](**json)
    
    def find_all_active_listings_by_shop(self, shop_id: int, limit: int = 25, sort_on: str = "created", sort_order: str = "desc", offset: int = 0, keywords: str = ""):
        endpoint = f"shops/{shop_id}/listings/active"
        
        params = {"limit": limit, "sort_on": sort_on, "sort_order": sort_order,
                  "offset": offset, "keywords": keywords
        }
        
        response = self.session.request(endpoint, params=params)
        return Response[Listing](**response)
        
    def get_listings_by_receipt_id(self, shop_id: int, receipt_id: int, limit: int = 25, offset: int = 0) -> Response[Listing]:
        endpoint = f"shops/{shop_id}/receipts/{receipt_id}/listings"
        params = {"limit": limit, "offset": offset}
        
        response = self.session.request(endpoint, params=params)
        return Response[Listing](**response)
    
    def get_listing_properties(self, shop_id: int, listing_id: int):
        endpoint = f"shops/{shop_id}/listings/{listing_id}/properties"
        
        response = self.session.request(endpoint)
        return Response[ListingProperty](**response)
    
    def get_listing_property(self, listing_id: int, property_id: int):
        endpoint = f"listings/{listing_id}/properties/{property_id}"
        
        response = self.session.request(endpoint)
        return ListingProperty(**response)
    
    def get_listing_inventory(self, listing_id: int):
        endpoint = f"listings/{listing_id}/inventory"
        response = self.session.request(endpoint)
        return Inventory(**response)
    
    def update_listing(self, shop_id: int, listing_id: int, **kwargs):
        endpoint = f"shops/{shop_id}/listings/{listing_id}"
        response = self.session.request(endpoint, "PATCH", data=kwargs)
        return Listing(**response)
    
    def update_listing_inventory(self, listing_id: int, products: List[dict], price_on_property: Optional[List[int]] = None, quantity_on_property: Optional[List[int]] = None, sku_on_property: Optional[List[int]] = None):
        
        data = {
            "products":products
        }

        if price_on_property is not None:
            data["price_on_property"] = price_on_property
        
        if quantity_on_property is not None:
            data["quantity_on_property"] = quantity_on_property
        
        if sku_on_property is not None:
            data["sku_on_property"] = sku_on_property
            
        endpoint = f"listings/{listing_id}/inventory"
        response = self.session.request(endpoint, "PUT", json=data)
        return Inventory(**response)

    def get_listings_by_listing_ids(self, listing_ids: List[int], includes: Optional[str] = None, _as="pydantic"):
        params = {
            "listing_ids":",".join(list(map(str, listing_ids))),
            "includes":includes
        }
        endpoint = "listings/batch"
        
        response = self.session.request(endpoint, params=params)
        if _as == "pydantic":
            
            return Response[Listing](**response)
        return response
    
    def get_listings_by_return_policy_id(self, return_policy_id: int, shop_id: int):
        endpoint = f"shops/{shop_id}/policies/return/{return_policy_id}/listings"
        response = self.session.request(endpoint)
        return Response[Listing](**response)
    
    def get_listings_by_shop_section_id(self, shop_id: int, shop_section_ids: List[int], limit: int = 25, offset: int = 0, sort_on: str = "created", sort_order: str = "desc"):
        endpoint = f"shops/{shop_id}/shop-sections/listings"
        params = {
            "shop_section_ids": shop_section_ids,
            "limit": limit,
            "offset": offset,
            "sort_on": sort_on,
            "sort_order": sort_order
        }
        
        response = self.session.request(endpoint, params=params)
        return Response[Listing](**response)
    
    def get_featured_listings_by_shop_id(self, shop_id: int, limit: int = 25, offset: int = 0):
        endpoint = f"shops/{shop_id}/listings/featured"
        params = {
            "limit": limit,
            "offset": offset
        }
        
        response = self.session.request(endpoint, params=params)
        
        return Response[Listing](**response)
    
    def get_listings_by_shop_id(self, shop_id: int, state: str = "active", limit: int = 25, offset: int = 0, sort_on: str = "created", sort_order: str = "desc", includes: List[str] = None) -> Response[Listing]:
            
        endpoint = f"shops/{shop_id}/listings"
        params = {
            "state": state,
            "limit": limit,
            "offset": offset,
            "sort_on": sort_on,
            "sort_order": sort_order,
        }
        if includes is not None:
            if isinstance(includes, str):
                
                params.update({"includes":includes})
            elif isinstance(includes, list):
                params.update({"includes":",".join(includes)})
                
        response = self.session.request(endpoint, params=params)
        return Response[Listing](**response)
    
    def get_listing_offering_by_id(self, listing_id: int, product_id: int, product_offering_id: int) -> Offering:
        endpoint = f"listings/{listing_id}/products/{product_id}/offerings/{product_offering_id}"
        response = self.session.request(endpoint)
        return Offering(**response)
    
    def get_listing_product_by_id(self, listing_id: int, product_id: int) -> Product:
        endpoint = f"listings/{listing_id}/inventory/products/{product_id}"
        response = self.session.request(endpoint)
        return Product(**response)
    
    def get_listing_translation(self, shop_id: int, listing_id: int, language: str) -> Translation:
        endpoint = f"shops/{shop_id}/listings/{listing_id}/translations/{language}"
        response = self.session.request(endpoint)
        return Translation(**response)
    
    def get_all_listing_files_by_id(self, shop_id: int, listing_id: int) -> Response[File]:
        endpoint = f"shops/{shop_id}/listings/{listing_id}/files"
        response = self.session.request(endpoint)
        return Response[File](**response)
    
    def get_listing_file_by_id(self, shop_id: int, listing_id: int, listing_file_id: int) -> File:
        endpoint = f"shops/{shop_id}/listings/{listing_id}/files/{listing_file_id}"
        response = self.session.request(endpoint)
        return File(**response)
    
    def upload_listing_file(self, shop_id: int, listing_id: int, listing_file_id: int, file: str, name: str, rank: int = 1):
        endpoint = f"shops/{shop_id}/listings/{listing_id}/files"
        response = self.session.request(endpoint, "POST", data={
            "listing_file_id": listing_file_id,
            "file": file,
            "name": name,
            "rank": rank
        })

        return File(**response)
    
    def delete_listing_file(self, shop_id: int, listing_id: int, listing_file_id: int) -> str:
        endpoint = f"shops/{shop_id}/listings/{listing_id}/files/{listing_file_id}"
        
        response = self.session.request(endpoint, method="DELETE")
        return response
    
    def delete_listing_by_id(self, listing_id: int) -> str:
        endpoint = f"listings/{listing_id}"
        response = self.session.request(endpoint, "DELETE")
        return response
    
    def get_listing_image_by_id(self, listing_id: int, listing_image_id: int) -> ListingImage:
        endpoint = f"listings/{listing_id}/images/{listing_image_id}"
        response = self.session.request(endpoint)
        return ListingImage(**response)
    
    def get_listing_images_by_id(self, listing_id: int) -> Response[ListingImage]:
        endpoint = f"listings/{listing_id}/images"
        response = self.session.request(endpoint)
        return Response[ListingImage](**response)
    
    def delete_listing_image_by_id(self, shop_id: int, listing_id: int, listing_image_id: int) -> str:
        endpoint = f"shops/{shop_id}/listings/{listing_id}/images/{listing_image_id}"
        response = self.session.request(endpoint, "DELETE")
        return response
    
    def get_listing_variation_images(self, shop_id: int, listing_id: int) -> Response[VariationImage]:
        endpoint = f"shops/{shop_id}/listings/{listing_id}/variation-images"
        response = self.session.request(endpoint)
        return Response[VariationImage](**response)
        
    def update_variation_images(self, shop_id: int, listing_id: int, variation_images: List[VariationImage]) -> Response[VariationImage]:
        endpoint = f"shops/{shop_id}/listings/{listing_id}/variation-images"
        data = {"variation_images": [image.dict() for image in variation_images]}
        response = self.session.request(endpoint, "POST", data=data)
        return Response[VariationImage](**response)
    
    def upload_listing_image(self, shop_id: int, listing_id: int, image: bytes, listing_image_id: int, rank: int = 1, overwrite: bool = False, is_watermarked: bool = False, alt_text: str = "") -> ListingImage:
        endpoint = f"shops/{shop_id}/listings/{listing_id}/images"
        data = {
            #"image": image,
            #"listing_image_id": None,
            "rank": rank,
            "overwrite": overwrite,
            "is_watermarked": is_watermarked,
            "alt_text": alt_text
        }
        print(type(image))
        response = self.session.request(endpoint, "POST", data=data, files={"image": image})
        
        
        return ListingImage(**response)
    
    def upload_multiple_images_to_listing(self, shop_id: int, listing_id: int, images: List[bytes], overwrite: bool = False, is_watermarked: bool = False, alt_text: str = "") -> List[ListingImage]:
        response = self.get_listing_images_by_id(listing_id=listing_id)
        new_listing_image_id = response.results[-1].listing_image_id + 1
        uploaded_images = []
        for image in images:
            uploaded_listing_image = self.upload_listing_image(
                shop_id=shop_id, listing_id=listing_id, image=image,
                listing_image_id=new_listing_image_id, rank=new_listing_image_id, overwrite=overwrite,
                is_watermarked=is_watermarked, alt_text=alt_text)
            uploaded_images.append(uploaded_listing_image)
            new_listing_image_id += 1
        
        return uploaded_images
    
    def get_listing_video(self, listing_id: int, video_id: int) -> Video:
        endpoint = f"listings/{listing_id}/videos/{video_id}"
        response = self.session.request(endpoint)
        return Video(**response)
    
    def get_listing_videos(self, listing_id: int) -> Response[Video]:
        endpoint = f"listings/{listing_id}/videos"
        response = self.session.request(endpoint)
        return Response[Video](**response)
    
    def upload_listing_video(self, shop_id: int, listing_id: int, video_id: int, video: bytes, name: str) -> Video:
        endpoint = f"shops/{shop_id}/listings/{listing_id}/videos"
        data = {
            "video_id": video_id,
            "video": video,
            "name": name
        }
        response = self.session.request(endpoint, "POST", data=data)
        return Video(**response)

    def delete_listing_video(self, shop_id: int, listing_id: int, video_id: int) -> str:
        endpoint = f"shops/{shop_id}/listings/{listing_id}/videos/{video_id}"
        response = self.session.request(endpoint, "DELETE")
        return response

    def create_draft_listing(self, shop_id: int, listing: DraftListing) -> Listing:
        endpoint = f"shops/{shop_id}/listings"
        response = self.session.request(endpoint, "POST", data=listing.dict(exclude_none=True))
        return Listing(**response)
    
    def get_buyer_taxonomy_nodes(self) -> Response[Taxonomy]:
        endpoint = "buyer-taxonomy/nodes"
        response = self.session.request(endpoint)
        return Response[Taxonomy](**response)
    
    def get_properties_by_buyer_taxonomy_id(self, taxonomy_id: int) -> ProductProperty:
        endpoint = f"buyer-taxonomy/nodes/{taxonomy_id}/properties"
        response = self.session.request(endpoint)
        return ProductProperty(**response)
    
    def get_seller_taxonomy_nodes(self):
        endpoint = "seller-taxonomy/nodes"
        response = self.session.request(endpoint)
        return Response[Taxonomy](**response)
    
    def get_properties_by_seller_taxonomy_id(self, taxonomy_id: int) -> ProductProperty:
        endpoint = f"seller-taxonomy/nodes/{taxonomy_id}/properties"
        response = self.session.request(endpoint)
        return ProductProperty(**response)
    
    async def aget_listing_by_id(self, listing_id: int, includes):
        endpoint = f"listings/{listing_id}"
        json = await self.session.async_request(endpoint, params={"includes":includes})
        return Listing(**json)
    
    async def afind_all_active_listings(self, limit=25, offset=0, keywords="", sort_on="created", sort_order="desc", min_price=None, max_price=None, taxonomy_id=None, shop_location="United States"):
        """
        Find All Active Listings By Keywords, Min price, Max Price Or Shop Location

        Args:
            limit (int, optional): Result Limit. Defaults to 25, Max 100.
            offset (int, optional): Result Offset. Defaults to 0.
            keywords (str, optional): Keywords For Active Listings. Defaults to "".
            sort_on (str, optional): Sort By Result. Defaults to "created".
            sort_order (str, optional): Sort By Order. Defaults to "desc".
            min_price (_type_, optional): Min Price. Defaults to None.
            max_price (_type_, optional): Max Price. Defaults to None.
            taxonomy_id (_type_, optional): Taxonomy Id. Defaults to None.
            shop_location (_type_, optional): Shop Location. Defaults to US.

        Returns:
            Response[Listing]: List of Listing Items
        """
        
        endpoint = f"listings/active"
        params = {
            "limit":limit, "offset":offset, "keywords":keywords, "sort_on":sort_on,
            "sort_order":sort_order, "min_price":min_price, "max_price":max_price,
            "taxonomy_id":taxonomy_id, "shop_location":shop_location
        }
        
        
        
        json = await self.session.async_request(endpoint, params=RequestEncodingMixin._encode_params(params))

        return Response[Listing](**json)
     
    async def aget_listings_by_listing_ids(self, session: AsyncOauth2Session, listing_ids: List[int], includes: str = ""):
        params = {
            "listing_ids":",".join(list(map(str, listing_ids))),
            "includes":includes
        }
        endpoint = "listings/batch"
        
        response = await self.session.async_request(endpoint, session=session, params=params)
        return Response[Listing](**response)


from dataclasses import dataclass
from typing import List, Union
from etsy_apiv3.utils import EtsySession, Response
from etsy_apiv3.models import Payment, PaymentAccountLedgerEntry
from etsy_apiv3.utils.EtsyOauth2Session import EtsyOauth2Session

@dataclass
class PaymentResource:
    
    session: Union[EtsySession, EtsyOauth2Session]
    
    def get_shop_payments(self, shop_id: int, payment_ids: List[int]) -> Response[Payment]:
        endpoint = f"shops/{shop_id}/payments"
        params = {
            "payment_ids": payment_ids
        }
        response = self.session.request(endpoint, params=params)
        return Response[Payment](**response)
    
    def get_payment_by_receipt_id(self, shop_id: int, receipt_id: int) -> Response[Payment]:
        endpoint = f"shops/{shop_id}/receipts/{receipt_id}/payments"
        response = self.session.request(endpoint)
        return Response[Payment](**response)
    
    def get_payment_by_account_ledger_entry_ids(self, shop_id: int, ledger_entry_ids: List[int]) -> Response[Payment]:
        endpoint = f"shops/{shop_id}/payment-account/ledger-entries/payments"
        params = {
            "ledger_entry_ids": ledger_entry_ids
        }
        response = self.session.request(endpoint, params=params)
        return Response[Payment](**response)
    
    def get_shop_payment_account_ledger_entry(self, shop_id: int, ledger_entry_id: int) -> PaymentAccountLedgerEntry:
        endpoint = f"shops/{shop_id}/payment-account/ledger-entries/{ledger_entry_id}"
        response = self.session.request(endpoint)
        return PaymentAccountLedgerEntry(**response)
    
    def get_shop_payment_account_ledger_entries(self, shop_id: int, min_created: int, max_created: int, limit: int = 25, offset: int = 0) -> Response[PaymentAccountLedgerEntry]:
        """_summary_

        Args:
            shop_id (int): Target Shop ID
            min_created (int): The earliest unix timestamp for when a record was created. 
            max_created (int): The latest unix timestamp for when a record was created.
            limit (int, optional): The maximum number of results to return. Defaults to 25.
            offset (int, optional): The number of records to skip before selecting the first result. Defaults to 0.
        Returns:
            Response[PaymentAccountLedgerEntry]: Response Object By PaymentAccountLedgerEntry
        """
        endpoint = f"shops/{shop_id}/payment-account/ledger-entries"
        response = self.session.request(endpoint)
        return Response[PaymentAccountLedgerEntry](**response)
    
    
    


from .ListingResource import ListingResource
from .ReceiptResource import ReceiptResource
from .ReviewResource import ReviewResource
from .ShopResource import ShopResource
from .PaymentResource import PaymentResource
